library mpwt_loading;

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:get/get.dart';

class MPWTLoading extends GetxController with GetTickerProviderStateMixin {
  final BuildContext context;
  MPWTLoading(this.context);

  OverlayEntry? entry;
  OverlayState? overlayState;

  RxDouble time = 0.0.obs;

  late AnimationController animationController;
  late AnimationController scaleController;
  late Animation<double> scaleAnimation;
  late AnimationController checkController;
  late Animation<double> checkAnimation;
  late Tween<double> tween;
  RxDouble tBegin = 0.0.obs;
  RxDouble tEnd = 1.0.obs;
  Color tcBegin = Colors.indigo;
  Color tcEnd = Colors.green;

  late Animation<Color?> animationColor;
  late Tween<Color> colorTween;

  OverlayEntry emptyOverlay = OverlayEntry(builder: (context) {
    return const SizedBox();
  });

  @override
  void onInit() {
    overlayState = Overlay.of(context);
    tween = Tween<double>(begin: tBegin.value, end: tEnd.value);
    colorTween = Tween<Color>(begin: tcBegin, end: tcEnd);
    animationController = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 2500));
    checkAnimation = CurvedAnimation(
        parent: animationController, curve: Curves.easeInOutCirc);
    tween.animate(checkAnimation);
    animationColor = ColorTween(begin: Colors.indigo, end: Colors.green)
        .animate(animationController);
    colorTween.animate(checkAnimation);
    animationController.forward();
    Timer.run(() {
      Timer.periodic(const Duration(milliseconds: 100), (timer) {
        time.value += 0.5;
      });
    });

    entry = OverlayEntry(builder: (context) {
      return GestureDetector(
        onTap: () {
          // entry!.remove();
        },
        child: Material(
          elevation: 0,
          color: Colors.transparent,
          child: Container(
            alignment: Alignment.center,
            decoration: BoxDecoration(
                color: Colors.transparent,
                borderRadius: BorderRadius.circular(15)),
            child: Container(
              width: 85,
              height: 85,
              padding: const EdgeInsets.all(10),
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(15),
                  color: Colors.grey.shade200),
              child: Stack(
                children: [
                  Center(
                    child: CircularProgressIndicator(
                          strokeWidth: 1.25,
                          color: Colors.indigo,
                          valueColor: animationController.drive(ColorTween(
                              begin: Colors.indigo, end: Colors.green)),
                        ),
                  ),
                  // Center(
                  //   child: Obx(() {
                  //     return time.value >= 1
                  //         ? SizeTransition(
                  //             sizeFactor: checkAnimation,
                  //             axis: Axis.horizontal,
                  //             child: Center(
                  //                 child: Icon(
                  //               CupertinoIcons.checkmark_alt,
                  //               color: animationColor.value,
                  //             )))
                  //         : const SizedBox();
                  //   }),
                  // ),
                  Align(alignment: Alignment.bottomCenter, child: Text('data'))
                ],
              ),
            ),
          ),
        ),
      );
    });

    //  WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
    //    Future.delayed(Duration.zero, () {
    //     overlayState!.insert(entry ?? emptyOverlay);
    //   });
    //  });

    // WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
    //   Future.delayed(const Duration(seconds: 3), () {
    //     entry!.remove();
    //     animationController.reset();
    //   });
    // });
    super.onInit();
  }

  void show({int? duration, String loadingString = ''}) {
    overlayState!.insert(entry ?? emptyOverlay);
    animationController.forward();
    checkAnimation = CurvedAnimation(
        parent: animationController, curve: Curves.easeInOutCirc);

    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      if (duration != null) {
        Future.delayed(Duration(seconds: duration), () {
          entry!.remove();
          animationController.reset();
          tBegin.value = 0;
          tEnd.value = 0;
          tcBegin = Colors.transparent;
          tcEnd = Colors.transparent;
          update();
        });
      }
    });
  }

  void removeLoading() {
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      entry!.remove();
          animationController.reset();
          tBegin.value = 0;
          tEnd.value = 0;
          tcBegin = Colors.transparent;
          tcEnd = Colors.transparent;
    });
  }
}

